/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.databinding.jaxb.tools;

import java.io.File;
import java.util.List;

import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.conf.JuliacConfig;

/**
 * This class class offers a static method to compile java source files.
 * <p>
 * It is based on OW2 Juliac library.
 * </p>
 * 
 * @author Mohammed EL JAI - EBM WebSourcing
 */
public class DataBindingCompiler {

	/**
	 * Compile the java source files existing in the directory given in parameter 
	 * @param baseDirPath : The base dire 
	 * @param srcs : The source directory name 
	 * @param classpaths : List of the class path 
	 * @throws CompilationException : It's thrown when a compilation problem is occurred 
	 */
	public static void compile(String baseDirPath, String srcs, List<String> classpaths) throws CompilationException {


		File baseDir = new File(baseDirPath).getAbsoluteFile();
		String s;


		/*
		 * Directory containing the generated source code. <gensrc>
		 */
		String genDirName = null;
		if (genDirName == null) {
			genDirName = "target/generated-sources/main/java";
		}

		/*
		 * Directory containing the result of the compilation of the generated
		 * source code. <genclass>
		 */
		String classDirName = null;
		if (classDirName == null) {
			classDirName = "target/classes";
		}

		/*
		 * Directory containing the packaged version of the generated code. <genjar>
		 */
		String jarDirName = null;
		if (jarDirName == null) {
			jarDirName = "target/classes";
		}

		/*
		 * Source root. <srcs>
		 */
		s = srcs;
		String[] src = s == null ? new String[0] : s.split(File.pathSeparator);

		s = "";
		String[] srclib = s == null ? new String[0] : s.split(File.pathSeparator);


		/*
		 * Preparing the classpath for Juliac. - the java.class.path system property
		 * because Spoon/JDT relies on it for loading referenced types - the
		 * classloader which is used by Juliac for loading resources (e.g. .fractal
		 * files)
		 */

		if (classpaths.size() != 0) {
			String newjcp = "";
			for (String classpath : classpaths) {
				if (newjcp.length() != 0) {
					newjcp += File.pathSeparatorChar;
				}
				newjcp += classpath;
			}
			System.setProperty("java.class.path", newjcp);  
		}
		
		try { 
			// Create juliac configurations to indicate the base dire, classpath ...
			Juliac jc = new Juliac();
			JuliacConfig juliacConfig = new JuliacConfig(jc);
			juliacConfig.setBaseDir(baseDir);
			juliacConfig.setGenDirName(genDirName);
			juliacConfig.setClassDirName(classDirName);
			juliacConfig.addSrcs(src);
			juliacConfig.addSrclibs(srclib);
			jc.setJuliacConfig(juliacConfig);
			// Call juliac to compile sources files
			jc.compile();
		}
		catch(Exception e) {
			throw new CompilationException(e);
		}
	}
}
